@@ -55,6 +55,7 @@ import xml.etree.ElementTree as ET
 from collections import OrderedDict
 import numpy as np
 import pyparsing
+import importlib
 
 
 if sys.platform == 'darwin':
@@ -788,7 +789,7 @@ class OMCSessionZMQ(OMCSessionHelper, OMCSessionBase):
 
 
 class ModelicaSystem(object):
-    def __init__(self, fileName=None, modelName=None, lmodel=[], useCorba=False, commandLineOptions=None, variableFilter=None, verbose=True):  # 1
+    def __init__(self, fileName=None, modelName=None, lmodel=[], useCorba=False, commandLineOptions=None, variableFilter=None, customBuildDirectory=None, verbose=True):  # 1
         """
         "constructor"
         It initializes to load file and build a model, generating object, exe, xml, mat, and json files. etc. It can be called :
@@ -856,10 +857,10 @@ class ModelicaSystem(object):
         self.getconn.sendExpression("setCommandLineOptions(\"--linearizationDumpLanguage=python\")")
         self.getconn.sendExpression("setCommandLineOptions(\"--generateSymbolicLinearization\")")
 
-        self.setTempDirectory()
+        self.setTempDirectory(customBuildDirectory)
 
         if fileName is not None:
-            self.loadFile()
+            self.loadFile(verbose)
             self.loadLibrary(verbose)
 
         ## allow directly loading models from MSL without fileName
@@ -879,11 +880,12 @@ class ModelicaSystem(object):
             if not cmdexp:
                 return print(self.getconn.sendExpression("getErrorString()"))
 
-    def loadFile(self):
+    def loadFile(self, verbose):
         # load file
         loadFileExp="".join(["loadFile(","\"",self.fileName,"\"",")"]).replace("\\","/")
         loadMsg = self.getconn.sendExpression(loadFileExp)
-        if not loadMsg:
+        ## Show notification or warnings to the user when verbose=True OR if some error occurred i.e., not result
+        if verbose or not loadMsg:
             return print(self.getconn.sendExpression("getErrorString()"))
 
     # for loading file/package, loading model and building model
@@ -893,37 +895,32 @@ class ModelicaSystem(object):
             if element is not None:
                 if isinstance(element, str):
                     if element.endswith(".mo"):
-                        loadModelResult = self.requestApi("loadFile", element)
-                        if not loadModelResult:
-                            loadmodelError = self.requestApi('getErrorString')
-                        ## always print the notification warning to user, to suppress the warnings add verbose=False
-                        if verbose:
-                            print(self.requestApi('getErrorString'))
+                        apiCall = "loadFile"
                     else:
-                        loadModelResult = self.requestApi("loadModel", element)
-                        if not loadModelResult:
-                            loadmodelError = self.requestApi('getErrorString')
-                        if verbose:
-                            print(self.requestApi('getErrorString'))
-
+                        apiCall = "loadModel"
+                    result = self.requestApi(apiCall, element)
                 elif isinstance(element, tuple):
                     if not element[1]:
                         libname = "".join(["loadModel(", element[0], ")"])
                     else:
                         libname = "".join(["loadModel(", element[0], ", ", "{", "\"", element[1], "\"", "}", ")"])
-                    loadModelResult = self.sendExpression(libname)
-                    if not loadModelResult:
-                        loadmodelError = self.sendExpression("getErrorString()")
-                    if verbose:
-                        print(self.requestApi('getErrorString'))
+                    result = self.sendExpression(libname)
                 else:
                     print("| info | loadLibrary() failed, Unknown type detected: ", element , " is of type ",  type(element), ", The following patterns are supported\n1)[\"Modelica\"]\n2)[(\"Modelica\",\"3.2.3\"), \"PowerSystems\"]\n")
+                ## Show notification or warnings to the user when verbose=True OR if some error occurred i.e., not result
+                if verbose or not result:
+                    print(self.requestApi('getErrorString'))
 
-    def setTempDirectory(self):
+    def setTempDirectory(self, customBuildDirectory):
         # create a unique temp directory for each session and build the model in that directory
-        self.tempdir = tempfile.mkdtemp()
-        if not os.path.exists(self.tempdir):
-            return print(self.tempdir, " cannot be created")
+        if customBuildDirectory is not None:
+            if not os.path.exists(customBuildDirectory):
+                print(customBuildDirectory, " does not exist")
+            self.tempdir = customBuildDirectory
+        else:
+            self.tempdir = tempfile.mkdtemp()
+            if not os.path.exists(self.tempdir):
+                print(self.tempdir, " cannot be created")
 
         exp="".join(["cd(","\"",self.tempdir,"\"",")"]).replace("\\","/")
         self.getconn.sendExpression(exp)
@@ -1229,8 +1226,12 @@ class ModelicaSystem(object):
             r=""
             self.resultfile = os.path.join(self.tempdir, self.modelName + "_res.mat").replace("\\", "/")
         else:
-            r=" -r=" + resultfile
-            self.resultfile = resultfile
+            if os.path.exists(resultfile):
+                r=" -r=" + resultfile
+                self.resultfile = resultfile
+            else:
+                r=" -r=" + os.path.join(self.tempdir, resultfile).replace("\\", "/")
+                self.resultfile = os.path.join(self.tempdir, resultfile).replace("\\", "/")
 
         # allow runtime simulation flags from user input
         if(simflags is None):
@@ -1279,6 +1280,7 @@ class ModelicaSystem(object):
         currentDir = os.getcwd()
         if (os.path.exists(getExeFile)):
             cmd = getExeFile + override + csvinput + r + simflags
+            cmd = cmd.split(" ")
             #print(cmd)
             os.chdir(self.tempdir)
             if (platform.system() == "Windows"):
@@ -1290,13 +1292,13 @@ class ModelicaSystem(object):
                     p = subprocess.Popen(cmd, env=my_env, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
                 else:
                     p = subprocess.Popen(cmd, env=my_env)
-                p.wait()
-                p.terminate()
             else:
                 if not verbose:
                     p = subprocess.Popen(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
                 else:
                     p = subprocess.Popen(cmd)
+            p.wait()
+            p.terminate()
             os.chdir(currentDir)
             self.simulationFlag = True
         else:
@@ -1766,11 +1768,10 @@ class ModelicaSystem(object):
             # this function is called from the generated python code linearized_model.py at runtime,
             # to improve the performance by directly reading the matrices A, B, C and D from the julia code and avoid building the linearized modelica model
             try:
-                ## add the generated linearfile directory to system path, as running from script does not find the module
+                ## do not add the linearfile directory to path, as multiple execution of linearization will always use the first added path, instead execute the file
                 ## https://github.com/OpenModelica/OMPython/issues/196
-                sys.path.append(os.path.dirname(linearFile))
-                from linearized_model import linearized_model
-                result = linearized_model()
+                module = importlib.machinery.SourceFileLoader("linearized_model", linearFile).load_module()
+                result = module.linearized_model()
                 (n, m, p, x0, u0, A, B, C, D, stateVars, inputVars, outputVars) = result
                 self.linearinputs = inputVars
                 self.linearoutputs = outputVars
